#    Copyright 2004 Andrew Wilkinson <aw@cs.york.ac.uk>.
#
#    This file is part of PyLinda (http://www-users.cs.york.ac.uk/~aw/pylinda)
#
#    PyLinda is free software; you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation; either version 2.1 of the License, or
#    (at your option) any later version.
#
#    PyLinda is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with PyLinda; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import threading

class Stats:
    def __init__(self):
        self.server_cons = 0
        self.user_cons = 0

        self.tuplespaces = 0
        self.ins = 0
        self.outs = 0

        self.semaphore = threading.Semaphore()

    def serverConnect(self):
        self.semaphore.acquire()
        self.server_cons += 1
        self.semaphore.release()
    def serverDisconnect(self):
        self.semaphore.acquire()
        self.server_cons -= 1
        self.semaphore.release()

    def userConnect(self):
        self.semaphore.acquire()
        self.user_cons += 1
        self.semaphore.release()
    def userDisconnect(self):
        self.semaphore.acquire()
        self.user_cons -= 1
        self.semaphore.release()

    def tuplespaceCreate(self):
        self.semaphore.acquire()
        self.tuplespaces += 1
        self.semaphore.release()
    def tuplespaceDestroy(self):
        self.semaphore.acquire()
        self.tuplespaces -= 1
        self.semaphore.release()

    def in_tuple(self):
        self.semaphore.acquire()
        self.ins += 1
        self.semaphore.release()
    def out_tuple(self):
        self.semaphore.acquire()
        self.outs += 1
        self.semaphore.release()

    def getStats(self):
        try:
            self.semaphore.acquire()
            r = {"servers": self.server_cons, "users": self.user_cons, "out": self.outs, "in": self.ins}
            self.semaphore.release()
        finally:
            return r
