#    Copyright 2004 Andrew Wilkinson <aw@cs.york.ac.uk>.
#
#    This file is part of PyLinda (http://www-users.cs.york.ac.uk/~aw/pylinda)
#
#    PyLinda is free software; you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation; either version 2.1 of the License, or
#    (at your option) any later version.
#
#    PyLinda is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with PyLinda; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

## \namespace tuplecontainer
## \brief This module contains the class the contains tuples
## \internal
##
## \author Andrew Wilkinson <aw@cs.york.ac.uk>
##

from __future__ import generators

class NoTuple(Exception):
    pass

class TupleContainer:
    def __init__(self):
        self.contain = {}

    def add(self, tup):
        if len(tup) == 1:
            if self.contain.has_key(tup[0]):
                self.contain[tup[0]][0] += 1
            else:
                self.contain[tup[0]] = [1, None] # TupleContainer()]
        else:
            if self.contain.has_key(tup[0]):
                if self.contain[tup[0]][1] is None:
                    self.contain[tup[0]] = (self.contain[tup[0]][0], TupleContainer())
                self.contain[tup[0]][1].add(tup[1:])
            else:
                self.contain[tup[0]] = [0, TupleContainer()]
                self.contain[tup[0]][1].add(tup[1:])

    def doesMatch(self, template, tup):
        if type(tup) != type(()):
            return False
        if len(template) != len(tup):
            return False

        for i in range(len(tup)):
            if type(template[i]) == type(()):
                if not self.doesMatch(template[i], tup[i]):
                    return False
            elif (template[i] != tup[i].__class__) and (template[i] != tup[i]):
                return False
        return True

    def matchOneTuple(self, template):
        return self.matchTuples(template).next()

    def matchTuples(self, template):
        m = self.doMatchTuples(template)
        while 1:
            yield m.next()

    def doMatchTuples(self, template):
        ele, template = template[0], template[1:]
        tups_list = self.contain.keys()

        if template == ():
            try:
                self.contain[ele]
            except KeyError:
                for t in tups_list:
                    if type(t) == type(()):
                        if self.doesMatch(ele, t):
                            for i in range(self.contain[t][0]):
                                yield(t, )
                    elif (ele == t.__class__) or (ele == t):
                        for i in range(self.contain[t][0]):
                            yield (t, )
            else:
                for i in range(self.contain[ele][0]):
                    yield (ele, )
        else:
            try:
                self.contain[ele]
            except KeyError:
                for t in tups_list:
                    if type(t) == type(()):
                        if self.doesMatch(ele, t) and self.contain[t][1] is not None:
                            m = self.contain[t][1].doMatchTuples(template)
                            while True:
                                try:
                                    yield (t,) + m.next()
                                except NoTuple:
                                    break
                    elif (ele == t.__class__) or (ele == t):
                        if self.contain[t][1] is not None:
                            m = self.contain[t][1].doMatchTuples(template)
                            while True:
                                try:
                                    yield (t,) + m.next()
                                except NoTuple:
                                    break
            else:
                if self.contain[ele][1] is not None:
                    m = self.contain[ele][1].doMatchTuples(template)
                    yield (ele, ) + m.next()
        raise NoTuple

    def delete(self, tup):
        try:
            if len(tup) == 1:
                self.contain[tup[0]][0] -= 1
            else:
                self.contain[tup[0]][1].delete(tup[1:])
            if self.contain[tup[0]][0] == 0 and (self.contain[tup[0]][1] is None or self.contain[tup[0]][1].isEmpty()):
                del self.contain[tup[0]]
        except KeyError:
            print "Error, deleting tuple %s, tuple does not exist" % (str(tup))

    def isEmpty(self):
        return len(self.contain) == 0
