#    Copyright 2004 Andrew Wilkinson <aw@cs.york.ac.uk>.
#
#    This file is part of PyLinda (http://www-users.cs.york.ac.uk/~aw/pylinda)
#
#    PyLinda is free software; you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation; either version 2.1 of the License, or
#    (at your option) any later version.
#
#    PyLinda is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with PyLinda; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import math
import struct
import sys
from convert import DowncastError, PackingError, UnknownFormat

if sys.byteorder == "little":
    byteorder = "L"
else:
    byteorder = "B"

try:
    size = int(math.ceil(math.log(sys.maxint, 2)+1))
except TypeError: # we're on a pre-2.3 system where log only takes one parameter
    size = int(math.ceil(((math.log10(sys.maxint) / math.log10(2))+1)))

def pack(value):
    if value >= 0:
        return "builtin.int(%i,B,F)%s" % (size, struct.pack("!I", value))
    else:
        return "builtin.int(%i,B,T)%s" % (size, struct.pack("!I", abs(value)))

fmt = {"L" : {"8": "<B", "16": "<H", "32": "<I", "64": "<Q"},
       "B" : {"8": ">B", "16": ">H", "32": ">I", "64": ">Q"}}

def unpack(data):
    assert data.startswith("builtin.int")

    end = data.find(")")
    size, endian, neg = data[12:end].split(",")

    if neg == "F":
        r = struct.unpack(fmt[endian][size], data[end+1:])[0]
    elif neg == "T":
        r = -struct.unpack(fmt[endian][size], data[end+1:])[0]
    else:
        raise UnknownFormat

    if r < sys.maxint and r > -sys.maxint:
        return int(r)

import convert
convert.registerType(long, "builtin.int", pack, unpack)
convert.registerType(int, "builtin.int", pack, unpack)
